import dotenv from 'dotenv';
import './types';
import fastify, { FastifyRequest } from 'fastify';
import mercurius from 'mercurius';
import mercuriusCodegen from 'mercurius-codegen';
import schema from './schema';
import { resolvers } from './resolvers';
import { PrismaClient } from '@prisma/client';
import fastifyCors from '@fastify/cors';
import { z } from 'zod';

dotenv.config({ path: '../../.env' });

const { allowedOrigins, port, host } = z
  .object({
    ALLOWED_ORIGINS: z.string(),
    API_HOST: z.string(),
    API_PORT: z.coerce.number(),
  })
  .transform((env) => {
    return {
      allowedOrigins: env.ALLOWED_ORIGINS.split(','),
      port: env.API_PORT,
      host: env.API_HOST,
    };
  })
  .parse(process.env);

const app = fastify({
  logger: true,
  exposeHeadRoutes: true,
});

const prisma = new PrismaClient();

const context = async (req: FastifyRequest) => {
  const userId = (req.headers['user-id'] as string) || null;
  return {
    prisma,
    userId,
  };
};

app.register(fastifyCors, {
  origin: (origin, callback) => {
    if (!origin || allowedOrigins.includes(origin)) return callback(null, true);
    return callback(new Error('Not allowed'), false);
  },
  methods: ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS'],
  allowedHeaders: ['Content-Type', 'Authorization', 'user-id'],
});

app.register(mercurius, {
  schema,
  subscription: true,
  context,
});

app.register(async ({ graphql }) => {
  resolvers.forEach((resolver) => {
    graphql.defineResolvers(resolver);
  });
});

app.get('/ping', async () => {
  return 'pong\n';
});

mercuriusCodegen(app, {
  targetPath: './src/generated/graphql.ts',
  codegenConfig: {
    scalars: {
      DateTime: 'Date',
    },
  },
});

app.listen({ host, port }, (err, address) => {
  if (err) {
    console.error(err);
    process.exit(1);
  }
  console.log(`Server listening at ${address}`);
});
